@@ -7,8 +7,10 @@ from pandas import DataFrame
 from sapiopylib.rest.pojo.DataRecordPaging import DataRecordPojoPageCriteria, DataRecordPojoListPageResult, \
     DataRecordPojoHierarchyPageCriteria, DataRecordPojoHierarchyListPageResult
 
-from sapiopylib.rest.User import SapioUser
+from sapiopylib.rest.User import SapioUser, SapioServerException
 from sapiopylib.rest.pojo.DataRecord import from_json_record_list, to_record_json_list, DataRecord
+from sapiopylib.rest.pojo.DataRecordSideLinkPaging import DataRecordSideLinkFromPageCriteria, \
+    DataRecordSideLinkFromListPageResult, DataRecordSideLinkToPageCriteria, DataRecordSideLinkToListPageResult
 from sapiopylib.rest.pojo.SapioAccessType import SapioAccessType
 
 
@@ -72,7 +74,7 @@ class DataRecordManager:
         """
         params = {'dataTypeName': data_type_name,
                   'dataFieldName': data_field_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecordmanager/querydatarecords'
         response = self.user.post(sub_path, params, value_list)
         self.user.raise_for_status(response)
@@ -93,7 +95,7 @@ class DataRecordManager:
         """
         record_id_list.sort()
         params = {'dataTypeName': data_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecordlist'
         response = self.user.post(sub_path, params=params, payload=record_id_list)
         self.user.raise_for_status(response)
@@ -109,7 +111,7 @@ class DataRecordManager:
         :return: The paged data
         """
         params = {'dataTypeName': data_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecordlist/all'
         response = self.user.get(sub_path, params)
         self.user.raise_for_status(response)
@@ -136,7 +138,7 @@ class DataRecordManager:
                                 record_id: int) -> Optional[DataRecord]:
         """
         Get the record of the given type that has the provided Record ID.
-        :param data_type_name: The Data Type Name of the record in the sytem.
+        :param data_type_name: The Data Type Name of the record in the system.
         :param record_id: The Record ID of the record in the system.
         :return: The data record if available. None if not found.
         """
@@ -151,7 +153,7 @@ class DataRecordManager:
         return DataRecord.from_json(json_dict)
 
     def get_parents(self, record_id: int, child_type_name: str, parent_type_name: str,
-                    paging_criteria: DataRecordPojoPageCriteria = None) -> DataRecordPojoListPageResult:
+                    paging_criteria: Optional[DataRecordPojoPageCriteria] = None) -> DataRecordPojoListPageResult:
         """
         Get the parents of the given type above the record with the given Record ID.
         :param record_id: The data type of the child record.
@@ -163,7 +165,7 @@ class DataRecordManager:
         params = {'recordId': record_id,
                   'childTypeName': child_type_name,
                   'parentTypeName': parent_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecord/parents'
         response = self.user.get(sub_path, params)
         self.user.raise_for_status(response)
@@ -171,7 +173,7 @@ class DataRecordManager:
         return DataRecordPojoListPageResult.from_json(json_dict)
 
     def get_parents_list(self, record_id_list: List[int], child_type_name: Optional[str], parent_type_name: str,
-                         paging_criteria: DataRecordPojoHierarchyPageCriteria = None) -> \
+                         paging_criteria: Optional[DataRecordPojoHierarchyPageCriteria] = None) -> \
             DataRecordPojoHierarchyListPageResult:
         """
         Get the parents of the given type above the records with the given Record IDs.
@@ -182,18 +184,37 @@ class DataRecordManager:
         :param paging_criteria: Optionally specify the page to retrieve for. Note the page size may be enforced in SaaS.
         :return: The current page's result.
         """
-        record_id_list.sort()
-        params = {'childTypeName': child_type_name,
-                  'parentTypeName': parent_type_name}
-        self.append_query_param(paging_criteria, params)
-        sub_path = '/datarecordlist/parents'
-        response = self.user.post(sub_path, params=params, payload=record_id_list)
-        self.user.raise_for_status(response)
-        json_dict = response.json()
-        return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+        try:
+            record_id_list.sort()
+            params = {'childTypeName': child_type_name,
+                      'parentTypeName': parent_type_name}
+            self._append_query_param(paging_criteria, params)
+            sub_path = '/datarecordlist/parents'
+            response = self.user.post(sub_path, params=params, payload=record_id_list)
+            self.user.raise_for_status(response)
+            json_dict = response.json()
+            return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+        except SapioServerException as e:
+            if e.client_error.response.status_code == 405:
+                # Method not found. Use the deprecated version to re-request
+                self.user.log_warn("""
+SAPIO PLATFORM VERSION MISMATCH. USING DEPRECATED WEBSERVICE CALL FORMAT.
+This is unsupported. Use at your own risk!
+                """)
+                params = {'recordIdList': ','.join(str(x) for x in record_id_list),
+                          'childTypeName': child_type_name,
+                          'parentTypeName': parent_type_name}
+                self._append_query_param(paging_criteria, params)
+                sub_path = '/datarecordlist/parents'
+                response = self.user.get(sub_path, params)
+                response.raise_for_status()
+                json_dict = response.json()
+                return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+            else:
+                raise e
 
     def get_children(self, record_id: int, child_type_name: str,
-                     paging_criteria: DataRecordPojoPageCriteria = None) -> DataRecordPojoListPageResult:
+                     paging_criteria: Optional[DataRecordPojoPageCriteria] = None) -> DataRecordPojoListPageResult:
         """
         Get the children of the given type below the record with the given Record ID.
         :param record_id: The Record ID of the parent record.
@@ -203,7 +224,7 @@ class DataRecordManager:
         """
         params = {'recordId': record_id,
                   'childTypeName': child_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecord/children'
         response = self.user.get(sub_path, params)
         self.user.raise_for_status(response)
@@ -211,7 +232,7 @@ class DataRecordManager:
         return DataRecordPojoListPageResult.from_json(json_dict)
 
     def get_children_list(self, record_id_list: List[int], child_type_name: str,
-                          paging_criteria: DataRecordPojoHierarchyPageCriteria = None) -> \
+                          paging_criteria: Optional[DataRecordPojoHierarchyPageCriteria] = None) -> \
             DataRecordPojoHierarchyListPageResult:
         """
         Get the children of the given type below the records with the given Record IDs.
@@ -219,17 +240,35 @@ class DataRecordManager:
         :param child_type_name: The data type name of the children to look for below the given Record IDs.
         :param paging_criteria: Optionally specify the page to retrieve for. Note the page size may be enforced in SaaS.
         """
-        record_id_list.sort()
-        params = {'childTypeName': child_type_name}
-        self.append_query_param(paging_criteria, params)
-        sub_path = '/datarecordlist/childrenbyid'
-        response = self.user.post(sub_path, params=params, payload=record_id_list)
-        self.user.raise_for_status(response)
-        json_dict = response.json()
-        return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+        try:
+            record_id_list.sort()
+            params = {'childTypeName': child_type_name}
+            self._append_query_param(paging_criteria, params)
+            sub_path = '/datarecordlist/childrenbyid'
+            response = self.user.post(sub_path, params=params, payload=record_id_list)
+            self.user.raise_for_status(response)
+            json_dict = response.json()
+            return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+        except SapioServerException as e:
+            if e.client_error.response.status_code == 405:
+                # Method not found. Use the deprecated version to re-request
+                self.user.log_warn("""
+SAPIO PLATFORM VERSION MISMATCH. USING DEPRECATED WEBSERVICE CALL FORMAT.
+This is unsupported. Use at your own risk!
+                """)
+                params = {'recordIdList': ','.join(str(x) for x in record_id_list),
+                          'childTypeName': child_type_name}
+                self._append_query_param(paging_criteria, params)
+                sub_path = '/datarecordlist/children'
+                response = self.user.get(sub_path, params)
+                response.raise_for_status()
+                json_dict = response.json()
+                return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
+            else:
+                raise e
 
     def get_ancestors(self, record_id: int, descendant_type_name: str, ancestor_type_name: str,
-                      paging_criteria: DataRecordPojoPageCriteria = None) -> DataRecordPojoListPageResult:
+                      paging_criteria: Optional[DataRecordPojoPageCriteria] = None) -> DataRecordPojoListPageResult:
         """
         Get the ancestors of the given type above the records with the given Record ID.
         :param record_id: The Record ID of the descendant record.
@@ -241,7 +280,7 @@ class DataRecordManager:
         params = {'recordId': record_id,
                   'descendantTypeName': descendant_type_name,
                   "ancestorTypeName": ancestor_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecord/ancestors'
         response = self.user.get(sub_path, params)
         self.user.raise_for_status(response)
@@ -249,7 +288,7 @@ class DataRecordManager:
         return DataRecordPojoListPageResult.from_json(json_dict)
 
     def get_ancestors_list(self, record_id_list: List[int], descendant_type_name: str, ancestor_type_name: str,
-                           paging_criteria: DataRecordPojoHierarchyPageCriteria = None) -> \
+                           paging_criteria: Optional[DataRecordPojoHierarchyPageCriteria] = None) -> \
             DataRecordPojoHierarchyListPageResult:
         """
         Get the ancestors of the given type above the records with the given Record IDs.
@@ -262,7 +301,7 @@ class DataRecordManager:
         record_id_list.sort()
         params = {'descendantTypeName': descendant_type_name,
                   "ancestorTypeName": ancestor_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecordlist/ancestors'
         response = self.user.post(sub_path, params=params, payload=record_id_list)
         self.user.raise_for_status(response)
@@ -270,7 +309,7 @@ class DataRecordManager:
         return DataRecordPojoHierarchyListPageResult.from_json(json_dict)
 
     def get_descendants(self, record_id: int, descendant_type_name: str,
-                        paging_criteria: DataRecordPojoPageCriteria = None) -> DataRecordPojoListPageResult:
+                        paging_criteria: Optional[DataRecordPojoPageCriteria] = None) -> DataRecordPojoListPageResult:
         """
         Get the descendants of the given type below the records with the given Record ID.
         :param record_id: The Record ID of the ancestor record.
@@ -280,7 +319,7 @@ class DataRecordManager:
         """
         params = {'recordId': record_id,
                   'descendantTypeName': descendant_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecord/descendants'
         response = self.user.get(sub_path, params)
         self.user.raise_for_status(response)
@@ -288,7 +327,7 @@ class DataRecordManager:
         return DataRecordPojoListPageResult.from_json(json_dict)
 
     def get_descendants_list(self, record_id_list: List[int], descendant_type_name: str,
-                             paging_criteria: DataRecordPojoHierarchyPageCriteria = None) -> \
+                             paging_criteria: Optional[DataRecordPojoHierarchyPageCriteria] = None) -> \
             DataRecordPojoHierarchyListPageResult:
         """
         Get the descendants of the given type below the records with the given Record IDs.
@@ -299,7 +338,7 @@ class DataRecordManager:
         """
         record_id_list.sort()
         params = {'descendantTypeName': descendant_type_name}
-        self.append_query_param(paging_criteria, params)
+        self._append_query_param(paging_criteria, params)
         sub_path = '/datarecordlist/descendants'
         response = self.user.post(sub_path, params=params, payload=record_id_list)
         self.user.raise_for_status(response)
@@ -648,10 +687,57 @@ class DataRecordManager:
             ret[parent_record] = record_id_list
         return ret
 
+    def get_side_link_to_list(self, data_record_list: List[DataRecord], linked_data_type_name: str,
+                              side_link_field_name: str, paging_criteria: Optional[DataRecordSideLinkToPageCriteria]
+                              = None) -> DataRecordSideLinkToListPageResult:
+        """
+        Get the side linked records of the given type that point back to the given records.
+        :param data_record_list: The list of records to retrieve side links to.
+        :param linked_data_type_name: The data type name of the side linked records.
+        :param side_link_field_name: The data field name on the linkedDataTypeName that points back to the given records.
+        :param paging_criteria: The current page to retrieve.
+        :return:  An object containing a map from the record key 'DataTypeName:RecordId' to arrays of data records that
+         are side linked to the provided list of Records as well as a criteria object that details how to retrieve the
+         next page of data.
+        """
+        data_record_list.sort()
+        params = {'linkedDataTypeName': linked_data_type_name,
+                  'sideLinkedFieldName': side_link_field_name}
+        self._append_query_param(paging_criteria, params)
+        sub_path = '/datarecordlist/sidelinksto'
+        request_body = [x.to_json() for x in data_record_list]
+        response = self.user.post(sub_path, params, request_body)
+        self.user.raise_for_status(response)
+        json_dict = response.json()
+        return DataRecordSideLinkToListPageResult.from_json(json_dict)
+
+    def get_side_link_from_list(self, data_record_list: List[DataRecord], side_link_field_name: str,
+                                paging_criteria: Optional[DataRecordSideLinkFromPageCriteria] = None) -> \
+            DataRecordSideLinkFromListPageResult:
+        """
+        Get the side linked records from the given records referenced by the given field name.
+        :param data_record_list: The list of records to retrieve side links.
+        :param side_link_field_name: The data field name on the given record that points to the records to retrieve.
+        :param paging_criteria: The current page to retrieve.
+        :return: An object containing a map from the record key 'DataTypeName:RecordId'
+        to arrays of data records that are side linked from the provided list of Records as well as a criteria object
+        that details how to retrieve the next page of data.
+        """
+        data_record_list.sort()
+        params = {'sideLinkedFieldName': side_link_field_name}
+        self._append_query_param(paging_criteria, params)
+        sub_path = '/datarecordlist/sidelinksfrom'
+        request_body = [x.to_json() for x in data_record_list]
+        response = self.user.post(sub_path, params, request_body)
+        self.user.raise_for_status(response)
+        json_dict = response.json()
+        return DataRecordSideLinkFromListPageResult.from_json(json_dict)
+
     @staticmethod
-    def append_query_param(paging_criteria: Union[None, DataRecordPojoPageCriteria,
-    DataRecordPojoHierarchyPageCriteria],
-                           params: dict):
+    def _append_query_param(paging_criteria: Union[
+        None, DataRecordPojoPageCriteria, DataRecordPojoHierarchyPageCriteria, DataRecordSideLinkFromPageCriteria,
+        DataRecordSideLinkToPageCriteria],
+                            params: dict):
         if paging_criteria is None:
             return
         if isinstance(paging_criteria, DataRecordPojoPageCriteria):
@@ -661,3 +747,10 @@ class DataRecordManager:
             params['lastSourceRecordId'] = paging_criteria.last_retrieved_source_record_id
             params['lastRetrievedRecordId'] = paging_criteria.last_retrieved_target_record_id
             params['pageSize'] = paging_criteria.page_size
+        if isinstance(paging_criteria, DataRecordSideLinkFromPageCriteria):
+            params['lastSourceRecordId'] = paging_criteria.last_retrieved_source_recordId
+            params['pageSize'] = paging_criteria.page_size
+        if isinstance(paging_criteria, DataRecordSideLinkToPageCriteria):
+            params['lastSourceRecordId'] = paging_criteria.last_retrieved_source_recordId
+            params['lastRetrievedRecordId'] = paging_criteria.last_retrieved_target_recordId
+            params['pageSize'] = paging_criteria.page_size
