@@ -1,6 +1,5 @@
 from typing import Any, List, Optional, Dict
 from enum import Enum
-from typing import Optional
 
 from sapiopylib.rest.pojo.datatype.FieldDefinition import FieldType
 from sapiopylib.rest.pojo.Sort import SortDirection, SortDirectionParser
@@ -25,6 +24,13 @@ class QueryRestriction(Enum):
 
 
 class CompositeTermOperation(Enum):
+    """
+    A composite report term describes an operation to be applied to one or two terms.
+
+    The operations possible are: AND, OR, NOT.
+
+    The operation "NOT" should not be used anymore, as there is a negated flag on each report term.
+    """
     AND_OPERATOR = 4, "AND", "AND", "AND"
     OR_OPERATOR = 5, "OR", "OR", "OR"
     NOT_OPERATOR = 6, "NOT", "NOT", "NOT"
@@ -45,6 +51,11 @@ class CompositeTermOperation(Enum):
 
 
 class RawTermOperation(Enum):
+    """
+    A raw term operation compares a term value with a raw value.
+
+    The possible operations are: >, <, ==, !=, >=, <=
+    """
     GREATER_THAN_OPERATOR = 0, ">", "&gt;", ">"
     LESS_THAN_OPERATOR = 1, "<", "&lt;", "<"
     EQUAL_TO_OPERATOR = 2, "=", "=", "="
@@ -69,7 +80,7 @@ class RawTermOperation(Enum):
 
 class TermType(Enum):
     """
-    The type of a custom report term.
+    The type of custom report term.
     RAW_TERM: you can think of this as a leaf node in the term tree.
 
     COMPOSITE_TERM: you can think of this as a parent of two terms.
@@ -84,7 +95,8 @@ class TermType(Enum):
     term_code: int
     internal_code: int
 
-    def __init__(self, internal_code, term_code):
+    def __init__(self, internal_code: int, term_code: int):
+        self.internal_code = internal_code
         self.term_code = term_code
 
     def get_term_code(self) -> int:
@@ -154,7 +166,11 @@ class RawReportTerm(AbstractReportTerm):
 
 class CompositeReportTerm(AbstractReportTerm):
     """
-     A composite custom report term describes a logical operation on two report terms.
+     A composite custom report term describes a logical operation on one or two report terms.
+
+     When operation is (AND, OR), we expect both children to be non-trivial.
+
+     The operation "NOT" should not be used anymore, as there is a negated flag on each report term.
     """
     term_operation: CompositeTermOperation
     left_child: AbstractReportTerm
@@ -318,10 +334,10 @@ class ExplicitJoinDefinition:
         """
         A custom join defined for an advanced search.
         :param data_type_name: The name of the (UNRELATED) data type to be joined to the other data types referenced
-        in the advanced search. The report term included in this object much define how this type will be joined with 
+        in the advanced search. The report term included in this object much define how this type will be joined with
         the other types in the search.
-        :param report_term: The terms that will be used to join the provided data type name to the advanced search 
-        that it is set on. These terms cannot reference data type names that have not been added to the advanced search 
+        :param report_term: The terms that will be used to join the provided data type name to the advanced search
+        that it is set on. These terms cannot reference data type names that have not been added to the advanced search
         yet.
         """
         self.data_type_name = data_type_name
