@@ -1,7 +1,5 @@
-from abc import ABC, abstractmethod
-from typing import Optional, Dict, Any, List
+from typing import List
 
-from sapiopylib.rest.pojo.chartdata.DashboardEnums import *
 from sapiopylib.rest.pojo.chartdata.DashboardSeries import *
 
 
