@@ -76,6 +76,22 @@ class AbstractVeloxFieldDefinition(ABC):
     audit_logged: bool = True
     default_table_column_width: Optional[int] = None
 
+    @property
+    def data_type_name(self) -> str:
+        """
+        Get the data type name of the current field.
+        This is the same as get_data_type_name()
+        """
+        return self._data_type_name
+
+    @property
+    def data_field_name(self) -> str:
+        """
+        Get the data field name of the current field.
+        This is the same as get_data_field_name()
+        """
+        return self._data_field_name
+
     def get_data_type_name(self):
         return self._data_type_name
 
@@ -831,7 +847,7 @@ class VeloxSideLinkFieldDefinition(AbstractVeloxFieldDefinition):
 
     def __init__(self, data_type_name: str, data_field_name: str, display_name: str,
                  default_value: Optional[int] = None, linked_data_type_name: Optional[str] = None, **kwargs):
-        super().__init__(data_type_name, FieldType.STRING, data_field_name, display_name, **kwargs)
+        super().__init__(data_type_name, FieldType.SIDE_LINK, data_field_name, display_name, **kwargs)
         self.default_value = default_value
         self.linked_data_type_name = linked_data_type_name
 
@@ -864,7 +880,7 @@ class VeloxActionStringFieldDefinition(AbstractVeloxFieldDefinition):
                  unique_value: bool = False, field_validator: Optional[FieldValidator] = None,
                  icon_name: Optional[str] = None, action_plugin_path: Optional[str] = None,
                  direct_edit: bool = False, **kwargs):
-        super().__init__(data_type_name, FieldType.STRING, data_field_name, display_name, **kwargs)
+        super().__init__(data_type_name, FieldType.ACTION_STRING, data_field_name, display_name, **kwargs)
         self.default_value = default_value
         self.max_length = max_length
         self.unique_value = unique_value
