@@ -1,14 +1,16 @@
-from typing import Optional, List, Dict, Any
-
-from sapiopylib.rest.pojo.datatype.FieldDefinition import AbstractVeloxFieldDefinition, FieldDefinitionParser
 import datetime
+from typing import List, Dict, Any
 
 from sapiopylib.rest.pojo.TableColumn import TableColumn, TableColumnParser
+from sapiopylib.rest.pojo.datatype.FieldDefinition import AbstractVeloxFieldDefinition, FieldDefinitionParser
 from sapiopylib.rest.pojo.eln.SapioELNEnums import *
 from sapiopylib.rest.utils.SapioDateUtils import java_millis_to_datetime
 
 
 class EntryAttachment:
+    """
+    Attachment metadata for an ELN attachment entry.
+    """
     attachment_type: EntryAttachmentType
     attachment_name: Optional[str]
 
@@ -18,6 +20,9 @@ class EntryAttachment:
 
 
 class EntryRecordAttachment(EntryAttachment):
+    """
+    Attachment metadata for an ELN attachment entry, when data of attachment is backed by a data record.
+    """
     record_id: Optional[int]
 
     def __init__(self, attachment_name: Optional[str] = None,
@@ -27,6 +32,9 @@ class EntryRecordAttachment(EntryAttachment):
 
 
 class EntryStaticAttachment(EntryAttachment):
+    """
+    Attachment metadata for an ELN attachment entry, when data of attachment is backed by template data.
+    """
     static_attachment_checksum: Optional[str]
 
     def __init__(self, attachment_name: Optional[str] = None,
@@ -36,6 +44,11 @@ class EntryStaticAttachment(EntryAttachment):
 
 
 class ExperimentEntry:
+    """
+    ELN Experiment Entry is a single widget of an ELN experiment.
+
+    The experiment entry can be a form, table, custom view, attachment, free-text, or a chart.
+    """
     field_definition_list: Optional[List[AbstractVeloxFieldDefinition]]
     entry_type: ElnEntryType
     data_type_name: Optional[str]
@@ -112,7 +125,7 @@ class ExperimentEntry:
 
 class ExperimentTextEntry(ExperimentEntry):
     """
-    An experiment entry type that contains a large block of text.
+    An experiment entry that contains a large block of text.
     """
     record_id: Optional[int]
 
@@ -124,7 +137,7 @@ class ExperimentTextEntry(ExperimentEntry):
 
 class ExperimentTempDataEntry(ExperimentEntry):
     """
-    An experiment entry type that displays custom data retrieved through a custom plugin to display data
+    An experiment entry that displays custom data retrieved through a custom plugin to display data
     that may or may not be backed by DataRecords.
     """
     plugin_path: Optional[str]
@@ -136,6 +149,9 @@ class ExperimentTempDataEntry(ExperimentEntry):
 
 
 class ExperimentTableEntry(ExperimentEntry):
+    """
+    An experiment entry that will display a table of data records.
+    """
     base_data_type: Optional[str]
     data_type_layout_name: Optional[str]
     field_set_id_list: Optional[List[int]]
@@ -179,6 +195,9 @@ class ExperimentTableEntry(ExperimentEntry):
 
 
 class ExperimentPluginEntry(ExperimentEntry):
+    """
+    An experiment entry that will display a custom client-side plugin.
+    """
     plugin_name: str
     using_template_data: bool
     provides_template_data: bool
@@ -192,6 +211,9 @@ class ExperimentPluginEntry(ExperimentEntry):
 
 
 class ExperimentFormEntry(ExperimentEntry):
+    """
+    An experiment entry that will display a form view of a single data record.
+    """
     form_name_list: Optional[List[str]]
     data_type_layout_name: Optional[str]
     base_data_type: Optional[str]
@@ -228,6 +250,9 @@ class ExperimentFormEntry(ExperimentEntry):
 
 
 class ExperimentDashboardEntry(ExperimentEntry):
+    """
+    An experiment entry that displays a chart.
+    """
     dashboard_guid: Optional[str]
     data_source_entry_id: Optional[int]
 
@@ -239,6 +264,9 @@ class ExperimentDashboardEntry(ExperimentEntry):
 
 
 class ExperimentAttachmentEntry(ExperimentEntry):
+    """
+    An experiment entry that displays attached data for download, and preview if file is available for preview.
+    """
     record_id: Optional[int]
     attachment_name: Optional[str]
     static_attachment_checksum: Optional[str]
