@@ -30,7 +30,8 @@ class MultiLayerPlatingManager:
         layer1 = layers[0]
 
         string = '{'
-        string += '    "dataTypeShownInDataTypePanel": "' + ','.join(map(lambda layer: layer.data_type_config.data_type_name, layers)) + '",'
+        string += '    "dataTypeShownInDataTypePanel": "' + ','.join(
+            map(lambda layer: layer.data_type_config.data_type_name, layers)) + '",'
         string += '    "modeType": "DILUTION",'
         string += '    "dataTypePreferencesJSONMap": {'
         string += '    },'
@@ -433,7 +434,8 @@ class _PlateRecordTracker:
         next_plate = next(self.plates, None)
         # create a plate if we are out of plates, we are allowed to overflow, and this layer is not set to repeat for
         # all current plates
-        if (next_plate is None and self.plate_config.overflow_to_new_plate and not self.repeat_until_all_plates) or self.create_first_plate:
+        if (next_plate is None and self.plate_config.overflow_to_new_plate and not self.repeat_until_all_plates) \
+                or self.create_first_plate:
             self.create_first_plate = False
             return self.create_plate(self.plate_config)
         else:
